/*******************************************************************************

    74C926 VIA SOFTWARE V1.01 

    C/ PIC 16F628A
    
    CONTADOR COM LATCH  PARA 4 DISPLAYS DE CTODO COMUM
    
    AUTOR: CLAUDIO LRIOS

    DATA: 02/10/2020

    EMULA FUNCIONALIDADES DO  CIRCUITO INTEGRADO MM74C926 USANDO UM PIC 16F628A

    OBSERVAES IMPORTANTES:
    ALGUNS PINOS TIVERAM QUE SER OBRIGATRIAMENTE ALTERADOS.
    O TEMPO DE RESPOSTA NO SER O MESMO QUE O CIRCUITO ORIGINAL DEVIDO A LATN-
    CIA DO FIRMWARE.
    NESTA VERSO, O TIMER 1  CONFIGURADO COMO CONTADOR. ELE RECEBE OS PULSOS DE
    CLOCK PELO PINO 12 DO PIC. ESTA CONTAGEM NUNCA  ALTERADA PELA ROTINA,
    EXCETO NO 'RESET'. QUANDO ESTE TIMER ULTRAPASSA 65535, UM VALOR  SOMADO A
    UM 'ACUMULADOR DE CONTINUIDADE'. O VALOR MOSTRADO NOS DISPLAYS EM MODO
    'CONTADOR'  A SOMA DO VALOR ATUAL DO TIMER 1 + ACUMULADOR DE CONTINUIDADE. 
    PARA A FUNO LATCH ENABLE, FOI USADO O MODULO CCP, NO MODO CAPTURA. QUANDO
    O VALOR  APRESENTADO NOS DISPLAYS EM 'MODO LATCH', O VALOR CAPTURADO NOS
    REGISTRADORES CCPR1H +CCPR1L  SOMADO TAMBEM COM O ACUMULADOR DE CONTINUIDA-
    DE.
    A CAPTURA OCORRE QUANDO O PINO 9 MUDA DE '1' PARA '0' (NA TRANSIO). SE O
    PINO 3 FOR COLOCADO EM NVEL '0' SER POSSVEL VER ESTE VALOR DEPOSITADO
    NO REGISTRADOR LATCH DE FORMA ESTTICA NOS DISPLAYS.
    MAS SE O PINO 3 ESTIVER EM '1' VEREMOS A CONTAGEM DOS PULSOS DE CLOCK NOS
    DISPLAYS DE FORMA DINMICA.
    APESAR DA ENTRADA 'RESET' ESTAR NO PINO 4, A FUNO ORIGINAL DO PIC (QUE
    RESETA COM '0') FOI SUBSTITUIDA POR UM RESET POR FIRMARE DE POLARIDADE
    INVERTIDA (COM '1').


    OBS. MONTAGEM  DE NATUREZA DIDTICA

    ESTE ARQUIVO  PARTE INTEGRANTE DO BLOG LARIOS.TECNOLOGIA.WS


*******************************************************************************/

#include <16F628A.h>
#use delay(clock=4000000)
#fuses NOWDT,INTRC_IO, NOPUT, NOPROTECT, NOBROWNOUT, NOMCLR, NOLVP, NOCPD

//BYTE
#byte PORTB = 0x06
#byte PORTA = 0x05
#byte TRISA = 0x85
#byte TRISB = 0x86
#byte CMCOM = 0x1f
#byte INTCON = 0X0B
#byte OPTION_REG = 0x81
#byte T1CON  = 0X10
#byte CCP1CON  = 0X17
#byte CCPR1H  = 0X16
#byte CCPR1L  = 0X15
#byte TMR1H  = 0X0F
#byte TMR1L  = 0X0E

//BIT
#bit  GIE    = 0X0B.7
#bit  PEIE   = 0X0B.6
#bit  T0IE   = 0X0B.5
#bit  T0IF   = 0X0B.2
#bit  CCP1IF = 0X0C.2
#bit  CCP1IE = 0X8C.2
#bit  TMR1IF   = 0X0C.0  //flag ado timer 1 overflow pir1.0
#bit  TMR1IE   = 0X8C.0  //flag de interrupo do timer 1 overflow pie1.0


#bit  seg_a = 0x5.6 //pino 15
#bit  seg_b = 0x5.7 //pino 16
#bit  seg_c = 0x5.0 //pino 17
#bit  seg_d = 0x5.2 //pino 1
#bit  seg_e = 0x5.3 //pino 2
#bit  seg_f = 0x6.0 //pino 6
#bit  seg_g = 0x6.7 //pino 13
#bit  Reset      = 0x5.5 //pino 4
#bit  Latch_En   = 0x6.3 //pino 9
#bit  Carry_Out  = 0x5.1 //pino 18
#bit  Disp_Sel   = 0x5.4 //pino 3
#bit  Clock      = 0x6.6 //pino 12
#bit  Aout  = 0x6.1  //pino 7
#bit  Bout  = 0x6.2  //pino 8
#bit  Cout  = 0x6.4  //pino 10
#bit  Dout  = 0x6.5  //pino 11


 //variveis globais
 int1 fmux=0;
 int8 mil=0,cent=0,dez=0,un=0;
 int8 th=0,th1=0,tl=0;
 int32 latch=0, contador=0, contador1=0, acc=0;
 /******************************************************************************
           ENVIA SEGUIMENTOS DO DISPLAYS
******************************************************************************/

void out_display( unsigned int8 val ){

   switch (val){

    case 0: {//nmero '0'
     seg_a=1;
     seg_b=1;
     seg_c=1;
     seg_d=1;
     seg_e=1;
     seg_f=1;
     seg_g=0;
     break;
    }
    case 1: {//nmero '1'
     seg_a=0;
     seg_b=1;
     seg_c=1;
     seg_d=0;
     seg_e=0;
     seg_f=0;
     seg_g=0;
     break;
    }
    case 2: {//nmero '2'
     seg_a=1;
     seg_b=1;
     seg_c=0;
     seg_d=1;
     seg_e=1;
     seg_f=0;
     seg_g=1;
     break;
    }
    case 3: {//nmero '3'
     seg_a=1;
     seg_b=1;
     seg_c=1;
     seg_d=1;
     seg_e=0;
     seg_f=0;
     seg_g=1;
     break;
    }
    case 4: {//nmero '4'
     seg_a=0;
     seg_b=1;
     seg_c=1;
     seg_d=0;
     seg_e=0;
     seg_f=1;
     seg_g=1;
     break;
    }
    case 5: {//nmero '5'
     seg_a=1;
     seg_b=0;
     seg_c=1;
     seg_d=1;
     seg_e=0;
     seg_f=1;
     seg_g=1;
     break;
    }
    case 6: {//nmero '6'
     seg_a=1;
     seg_b=0;
     seg_c=1;
     seg_d=1;
     seg_e=1;
     seg_f=1;
     seg_g=1;
     break;
    }
    case 7: {//nmero '7'
     seg_a=1;
     seg_b=1;
     seg_c=1;
     seg_d=0;
     seg_e=0;
     seg_f=0;
     seg_g=0;
     break;
    }
    case 8: {//nmero '8'
     seg_a=1;
     seg_b=1;
     seg_c=1;
     seg_d=1;
     seg_e=1;
     seg_f=1;
     seg_g=1;
     break;
    }
    case 9: {//nmero '9'
     seg_a=1;
     seg_b=1;
     seg_c=1;
     seg_d=1;
     seg_e=0;
     seg_f=1;
     seg_g=1;
     break;
    }
   }//switch (v)
}


 /******************************************************************************
           MULTIPLEXA OS DISPLAYS
******************************************************************************/

 void multiplexa(){
    static int8 vez;

   if(++vez>3) vez=0;
    Aout=0;//desliga todos os displays
    Bout=0;
    Cout=0;
    Dout=0;

    switch (vez){

     case 0: {
       out_display(mil);
       Aout=1;
       break;}

     case 1: {
       out_display(cent);
       Bout=1;
       break;}

     case 2: {
       out_display(dez);
       Cout=1;
       break;}

     case 3: {
       out_display(un);
       Dout=1;
       break;}
    }
 }

/******************************************************************************
           OBTEM VALOR A SER APRESENTADO NOS DISPLAYS
******************************************************************************/
void get_dado(){

    mil=0;//zera acumuladores de resultado p/ milhar, centena, dezena e unidade
    cent=0;
    dez=0;
    un=0;
    /* Leitura do timer 1 sem desliga-lo ou alterar seu valor.
    Isto exige um teste de qualidade da leitura, pois pode ocorrer que o valor
    de tmr1h seja alterado durante o processo de leitura.
    */
    th=TMR1H;//faz leitura do timer 1 high
    tl=TMR1L;//faz leitura do timer 1 low
    th1=TMR1H;//faz leitura do timer 1 high novamente para comparar

    if(th==th1){//leitura correta do timer1H
      contador = (int32)th<<8;
      contador=  contador + tl;
    }
    else{//houve alterao no tmr1h
      contador = (int32)TMR1H<<8;//faz nova leitura
      contador= contador + TMR1L;
    }
    //testa se houve um overflow no timer 1
    if(TMR1IF==1){//Houve, ento:
     acc=acc+5536;//soma valor (65536->reduzindo multiplos 10000 sobra 5536)
     TMR1IF=0;//reseta flag
    }
    
    /*
    O timer 1 consegue acumular at 6 x a contagem 9999. Portanto, os multi-
    plos de 10000 podem ser cancelados, uma vez que no sero mostrados nos
    displays
    */
    if(acc>10000)acc=acc-10000;//reduz multiplos de 10000
    contador=contador+acc;//soma acumulador de 'continuidade' de contagem
    while(contador>=10000)contador=contador-10000;//reduz multiplos
    if(contador>5999)Carry_Out=1; else Carry_Out=0;//sada Carry_out

    //verifica qual a apresentao desejada pelo usurio (pino 3)
    if(Disp_Sel==0){//mostra valor capturado no latch nos displays (esttico)
      contador1=latch;
    }
    else{
      contador1=contador;//mostra contagem nos displays (dinmico, mas no em
                         //tempo real)
    }
    //obtm valores para os digitos de milhar, centena, dezena e unidade
    while(contador1>=10000)contador1=contador1-10000;//reduz multiplos
    while(contador1>=1000){contador1=contador1 - 1000; mil++;}//milhar
    while(contador1>=100){contador1=contador1 - 100; cent++;}//centena
    while(contador1>=10){contador1=contador1 - 10; dez++;}//dezena
    un=contador1;//unidade

}
/******************************************************************************
             ROTINAS DE INTERRUPES
******************************************************************************/

#INT_CCP1
void INT_CCP1_ISR(){//interrupo da 'captura'
      if(TMR1IF==1){//ser que houve estouro do timer 1? se sim:
       acc=acc+5536;//soma valor (65536->reduzindo multiplos 10000 sobra 5536)
       TMR1IF=0;//reseta flag
      }
     latch= (int32)CCPR1H<<8;//pega valor high capturado
     latch= latch + CCPR1L;//pega valor low capturado
     latch= latch+acc;//soma contador de continuidade de contagem
     CCP1IF=0; //apaga flag do ccp1
}

#INT_TIMER0
void INT_TIMER0_ISR(){//interrupo do timer 0 (cada 1024 useg)
   fmux=1;//sinaliza para ocorrer uma multiplexao dos displays
 }

//==============================================================================
//         ROTINA MAIN
//==============================================================================

void main() {

   OPTION_REG = 0B00000001; //timer0 div/4 (usado na multiplexao dos displays)
   CMCOM=7; //desliga comparadores analgicos
   TRISA=0b00110000;//sentido fixo das portas
   TRISB=0B01001000;
   PORTA=0;//desliga displays (catodo comum)
   PORTB=0;
   T1CON= 0B00000011;//timer 1 sem prescaller clock ext.com SYNC - pino 12 (RB6)
   TMR1IF=0;
   CCP1CON= 0B00000100; //capture mode falling edge
   CCP1IF=0;
   CCP1IE=1;//liga interrupo de 'capture mode'
   T0IE=1;//liga interrupo do timer 0 overflow
   PEIE=1;//liga interrupo de perifricos
   GIE=1;//liga geral das interrupes

//==============================================================================
//         LAO DE REPETIO PRINCIPAL
//==============================================================================

   for(;;){

      if(Reset==1){//testa boto de reset
         Carry_Out=0;//zera sada de carry out
         TMR1L=0;//zera timer1 low
         TMR1H=0;//zera timer1 high
         TMR1IF=0;//reseta flag do timer1
         acc=0;//zera acumulador de continuidade
      }
      if(fmux==1){//multiplexa os 4 displays de 7 seguimentos
         get_dado();//l a contagem
         multiplexa();//mostra nos displays
         fmux=0;//reseta flag
      }

   }
}
//==========================fim da rotina=======================================
